=== modified file 'plugin/innobase/fil/fil0fil.cc'
--- plugin/innobase/fil/fil0fil.cc	2011-04-07 12:10:16 +0000
+++ plugin/innobase/fil/fil0fil.cc	2011-07-30 09:20:56 +0000
@@ -641,7 +641,7 @@
 	fil_system_t*	system,	/*!< in: tablespace memory cache */
 	fil_space_t*	space)	/*!< in: space */
 {
-	ib_int64_t	size_bytes;
+	uint64_t	size_bytes;
 	ulint		size_low;
 	ulint		size_high;
 	ibool		ret;
@@ -683,11 +683,10 @@
 
 		os_file_get_size(node->handle, &size_low, &size_high);
 
-		size_bytes = (((ib_int64_t)size_high) << 32)
-			+ (ib_int64_t)size_low;
+		size_bytes = (((uint64_t)size_high) << 32) + size_low;
 #ifdef UNIV_HOTBACKUP
 		if (space->id == 0) {
-			node->size = (ulint) (size_bytes / UNIV_PAGE_SIZE);
+			node->size = size_bytes / UNIV_PAGE_SIZE;
 			os_file_close(node->handle);
 			goto add_size;
 		}
@@ -764,7 +763,7 @@
 		}
 
 		if (!(flags & DICT_TF_ZSSIZE_MASK)) {
-			node->size = (ulint) (size_bytes / UNIV_PAGE_SIZE);
+			node->size = (ulint)size_bytes / UNIV_PAGE_SIZE;
 		} else {
 			node->size = (ulint)
 				(size_bytes
@@ -3186,7 +3185,7 @@
 	ulint		flags;
 	ulint		size_low;
 	ulint		size_high;
-	ib_int64_t	size;
+	uint64_t	size;
 #ifdef UNIV_HOTBACKUP
 	fil_space_t*	space;
 #endif
@@ -4305,9 +4304,6 @@
 	ut_ad(ut_is_2pow(zip_size));
 	ut_ad(buf);
 	ut_ad(len > 0);
-#if (1 << UNIV_PAGE_SIZE_SHIFT) != UNIV_PAGE_SIZE
-# error "(1 << UNIV_PAGE_SIZE_SHIFT) != UNIV_PAGE_SIZE"
-#endif
 	ut_ad(fil_validate());
 #ifndef UNIV_HOTBACKUP
 # ifndef UNIV_LOG_DEBUG

=== modified file 'plugin/innobase/fsp/fsp0fsp.cc'
--- plugin/innobase/fsp/fsp0fsp.cc	2010-12-27 18:39:11 +0000
+++ plugin/innobase/fsp/fsp0fsp.cc	2011-04-05 10:32:12 +0000
@@ -650,16 +650,12 @@
 				0 for uncompressed pages */
 	ulint	offset)		/*!< in: page offset */
 {
-#ifndef DOXYGEN /* Doxygen gets confused of these */
-# if UNIV_PAGE_SIZE <= XDES_ARR_OFFSET \
-		+ (UNIV_PAGE_SIZE / FSP_EXTENT_SIZE) * XDES_SIZE
-#  error
-# endif
-# if PAGE_ZIP_MIN_SIZE <= XDES_ARR_OFFSET \
-		+ (PAGE_ZIP_MIN_SIZE / FSP_EXTENT_SIZE) * XDES_SIZE
-#  error
-# endif
-#endif /* !DOXYGEN */
+	ut_a(UNIV_PAGE_SIZE >
+	     XDES_ARR_OFFSET + (UNIV_PAGE_SIZE / FSP_EXTENT_SIZE) * XDES_SIZE);
+	ut_a(PAGE_ZIP_MIN_SIZE >
+	     XDES_ARR_OFFSET +
+	     (PAGE_ZIP_MIN_SIZE / FSP_EXTENT_SIZE) * XDES_SIZE);
+
 	ut_ad(ut_is_2pow(zip_size));
 
 	if (!zip_size) {
@@ -1452,13 +1448,6 @@
 							   mtr);
 		xdes_init(descr, mtr);
 
-#if UNIV_PAGE_SIZE % FSP_EXTENT_SIZE
-# error "UNIV_PAGE_SIZE % FSP_EXTENT_SIZE != 0"
-#endif
-#if PAGE_ZIP_MIN_SIZE % FSP_EXTENT_SIZE
-# error "PAGE_ZIP_MIN_SIZE % FSP_EXTENT_SIZE != 0"
-#endif
-
 		if (UNIV_UNLIKELY(init_xdes)) {
 
 			/* The first page in the extent is a descriptor page

=== modified file 'plugin/innobase/handler/ha_innodb.cc'
--- plugin/innobase/handler/ha_innodb.cc	2011-07-25 14:24:19 +0000
+++ plugin/innobase/handler/ha_innodb.cc	2011-07-30 09:20:56 +0000
@@ -183,6 +183,14 @@
 static buffer_pool_constraint innobase_buffer_pool_size;
 typedef constrained_check<uint32_t, MAX_BUFFER_POOLS, 1> buffer_pool_instances_constraint;
 static buffer_pool_instances_constraint innobase_buffer_pool_instances;
+typedef constrained_check<uint32_t,
+			  (1 << UNIV_PAGE_SIZE_SHIFT_MAX),
+				(1 << 12)> page_size_constraint;
+static page_size_constraint innobase_page_size;
+typedef constrained_check<uint32_t,
+			  (1 << UNIV_PAGE_SIZE_SHIFT_MAX),
+				(1 << 9)> log_block_size_constraint;
+static log_block_size_constraint innobase_log_block_size;
 typedef constrained_check<uint32_t, UINT32_MAX, 100> io_capacity_constraint;
 static io_capacity_constraint innodb_io_capacity;
 typedef constrained_check<uint32_t, 5000, 1> purge_batch_constraint;
@@ -2218,6 +2226,67 @@
   }
 #endif /* UNIV_DEBUG */
 
+  srv_page_size = 0;
+  srv_page_size_shift = 0;
+
+  uint32_t page_size = innobase_page_size.get();
+  uint32_t log_block_size = innobase_log_block_size.get();
+
+  if (innobase_page_size != (1 << 14)) {
+    uint n_shift;
+
+    errmsg_printf(error::WARN,
+		  "InnoDB: Warning: innodb_page_size has been changed from default value 16384. (###EXPERIMENTAL### operation)\n");
+    for (n_shift = 12; n_shift <= UNIV_PAGE_SIZE_SHIFT_MAX; n_shift++) {
+      if (innobase_page_size == (1UL << n_shift)) {
+	srv_page_size_shift = n_shift;
+	srv_page_size = (1 << srv_page_size_shift);
+	errmsg_printf(error::WARN,
+		      "InnoDB: The universal page size of the database is set to %lu.\n",
+		      srv_page_size);
+	break;
+      }
+    }
+  } else {
+    srv_page_size_shift = 14;
+    srv_page_size = (1 << srv_page_size_shift);
+  }
+
+  if (!srv_page_size_shift) {
+    errmsg_printf(error::ERROR,
+		  "InnoDB: Error: %"PRIu32" is not a valid value for innodb_page_size.\n"
+		  "InnoDB: Error: Valid values are 4096, 8192, and 16384 (default=16384).\n",
+		  page_size);
+    goto error;
+  }
+
+  srv_log_block_size = 0;
+  if (log_block_size != (1 << 9)) { /*!=512*/
+    uint	n_shift;
+
+    errmsg_printf(error::WARN,
+		  "InnoDB: Warning: innodb_log_block_size has been changed from default value 512. (###EXPERIMENTAL### operation)\n");
+    for (n_shift = 9; n_shift <= UNIV_PAGE_SIZE_SHIFT_MAX; n_shift++) {
+      if (log_block_size == (1UL << n_shift)) {
+	srv_log_block_size = (1 << n_shift);
+	errmsg_printf(error::WARN, "InnoDB: The log block size is set to %"PRIu32".\n",
+		      srv_log_block_size);
+	break;
+      }
+    }
+  } else {
+    srv_log_block_size = 512;
+  }
+
+  if (!srv_log_block_size) {
+    errmsg_printf(error::ERROR,
+		  "InnoDB: Error: %"PRIu32" is not a valid value for innodb_log_block_size.\n"
+		  "InnoDB: Error: A valid value for innodb_log_block_size is\n"
+		  "InnoDB: Error: a power of 2 from 512 to 16384.\n",
+		  log_block_size);
+    goto error;
+  }
+
   os_innodb_umask = (ulint)internal::my_umask;
 
 
@@ -2495,6 +2564,8 @@
                                                   innodb_file_format_max_validate));
   context.registerVariable(new sys_var_constrained_value_readonly<size_t>("buffer_pool_size", innobase_buffer_pool_size));
   context.registerVariable(new sys_var_constrained_value_readonly<int64_t>("log_file_size", innobase_log_file_size));
+  context.registerVariable(new sys_var_constrained_value_readonly<uint32_t>("page_size", innobase_page_size));
+  context.registerVariable(new sys_var_constrained_value_readonly<uint32_t>("log_block_size", innobase_log_block_size));
   context.registerVariable(new sys_var_constrained_value_readonly<uint32_t>("flush_log_at_trx_commit",
                                                   innodb_flush_log_at_trx_commit));
   context.registerVariable(new sys_var_constrained_value_readonly<unsigned int>("max_dirty_pages_pct",
@@ -6269,9 +6340,6 @@
       | DICT_TF_COMPACT
       | DICT_TF_FORMAT_ZIP
       << DICT_TF_FORMAT_SHIFT;
-#if DICT_TF_ZSSIZE_MAX < 1
-# error "DICT_TF_ZSSIZE_MAX < 1"
-#endif
 
     if (strict_mode)
     {
@@ -9352,6 +9420,12 @@
   context("log-file-size",
           po::value<log_file_constraint>(&innobase_log_file_size)->default_value(20*1024*1024L),
           "The size of the buffer which InnoDB uses to write log to the log files on disk.");
+  context("page-size",
+	  po::value<page_size_constraint>(&innobase_page_size)->default_value(1 << 14),
+	  "###EXPERIMENTAL###: The universal page size of the database. Changing for created database is not supported. Use on your own risk!");
+  context("log-block-size",
+	  po::value<log_block_size_constraint>(&innobase_log_block_size)->default_value(1 << 9),
+	  "###EXPERIMENTAL###: The log block size of the transaction log file. Changing for created log file is not supported. Use on your own risk!");
   context("log-files-in-group",
           po::value<log_files_in_group_constraint>(&innobase_log_files_in_group)->default_value(2),
           "Number of log files in the log group. InnoDB writes to the files in a circular fashion.");

=== modified file 'plugin/innobase/include/buf0buf.h'
--- plugin/innobase/include/buf0buf.h	2011-03-14 05:40:28 +0000
+++ plugin/innobase/include/buf0buf.h	2011-04-05 10:32:12 +0000
@@ -1560,7 +1560,7 @@
 	time_t		last_printout_time;
 					/*!< when buf_print_io was last time
 					called */
-	buf_buddy_stat_t buddy_stat[BUF_BUDDY_SIZES + 1];
+	buf_buddy_stat_t buddy_stat[BUF_BUDDY_SIZES_MAX + 1];
 					/*!< Statistics of buddy system,
 					indexed by block size */
 	buf_pool_stat_t	stat;		/*!< current statistics */
@@ -1656,7 +1656,7 @@
 	/* @{ */
 	UT_LIST_BASE_NODE_T(buf_page_t)	zip_clean;
 					/*!< unmodified compressed pages */
-	UT_LIST_BASE_NODE_T(buf_page_t) zip_free[BUF_BUDDY_SIZES];
+	UT_LIST_BASE_NODE_T(buf_page_t) zip_free[BUF_BUDDY_SIZES_MAX];
 					/*!< buddy free lists */
 
 	buf_page_t			watch[BUF_POOL_WATCH_SIZE];
@@ -1664,9 +1664,6 @@
 					pool watches. Protected by
 				       	buf_pool->mutex. */
 
-#if BUF_BUDDY_HIGH != UNIV_PAGE_SIZE
-# error "BUF_BUDDY_HIGH != UNIV_PAGE_SIZE"
-#endif
 #if BUF_BUDDY_LOW > PAGE_ZIP_MIN_SIZE
 # error "BUF_BUDDY_LOW > PAGE_ZIP_MIN_SIZE"
 #endif

=== modified file 'plugin/innobase/include/buf0types.h'
--- plugin/innobase/include/buf0types.h	2011-03-14 05:40:28 +0000
+++ plugin/innobase/include/buf0types.h	2011-04-05 10:32:12 +0000
@@ -74,6 +74,7 @@
 					sizeof(buf_page_t) */
 #define BUF_BUDDY_SIZES		(UNIV_PAGE_SIZE_SHIFT - BUF_BUDDY_LOW_SHIFT)
 					/*!< number of buddy sizes */
+#define BUF_BUDDY_SIZES_MAX	(UNIV_PAGE_SIZE_SHIFT_MAX - BUF_BUDDY_LOW_SHIFT)
 
 /** twice the maximum block size of the buddy system;
 the underlying memory is aligned by this amount:

=== modified file 'plugin/innobase/include/fsp0types.h'
--- plugin/innobase/include/fsp0types.h	2011-03-14 05:40:28 +0000
+++ plugin/innobase/include/fsp0types.h	2011-04-05 10:32:12 +0000
@@ -43,7 +43,7 @@
 /* @} */
 
 /** File space extent size (one megabyte) in pages */
-#define	FSP_EXTENT_SIZE		(1 << (20 - UNIV_PAGE_SIZE_SHIFT))
+#define	FSP_EXTENT_SIZE		(1U << (20 - UNIV_PAGE_SIZE_SHIFT))
 
 /** On a page of any file segment, data may be put starting from this
 offset */

=== modified file 'plugin/innobase/include/ibuf0ibuf.ic'
--- plugin/innobase/include/ibuf0ibuf.ic	2010-12-18 04:43:40 +0000
+++ plugin/innobase/include/ibuf0ibuf.ic	2011-04-05 10:32:12 +0000
@@ -305,9 +305,6 @@
 	before = ibuf_index_page_calc_free_bits(0, max_ins_size);
 
 	if (max_ins_size >= increase) {
-#if ULINT32_UNDEFINED <= UNIV_PAGE_SIZE
-# error "ULINT32_UNDEFINED <= UNIV_PAGE_SIZE"
-#endif
 		after = ibuf_index_page_calc_free_bits(0, max_ins_size
 						       - increase);
 #ifdef UNIV_IBUF_DEBUG

=== modified file 'plugin/innobase/include/log0log.h'
--- plugin/innobase/include/log0log.h	2011-03-14 05:40:28 +0000
+++ plugin/innobase/include/log0log.h	2011-04-05 10:32:12 +0000
@@ -673,6 +673,9 @@
 					when mysqld is first time started
 					on the restored database, it can
 					print helpful info for the user */
+#define LOG_FILE_OS_FILE_LOG_BLOCK_SIZE	64
+					/* The log_block_size of XtraDB.
+					0 means default 512. */
 #define	LOG_FILE_ARCH_COMPLETED	OS_FILE_LOG_BLOCK_SIZE
 					/* this 4-byte field is TRUE when
 					the writing of an archived log file

=== modified file 'plugin/innobase/include/os0file.h'
--- plugin/innobase/include/os0file.h	2011-03-14 05:40:28 +0000
+++ plugin/innobase/include/os0file.h	2011-04-05 10:32:12 +0000
@@ -101,7 +101,7 @@
 if this fails for a log block, then it is equivalent to a media failure in the
 log. */
 
-#define OS_FILE_LOG_BLOCK_SIZE		512
+#define OS_FILE_LOG_BLOCK_SIZE		srv_log_block_size
 
 /** Options for file_create @{ */
 #define	OS_FILE_OPEN			51
@@ -190,6 +190,8 @@
 extern ulint	os_n_file_writes;
 extern ulint	os_n_fsyncs;
 
+extern uint32_t srv_log_block_size;
+
 #ifdef UNIV_PFS_IO
 /* Keys to register InnoDB I/O with performance schema */
 extern mysql_pfs_key_t	innodb_file_data_key;

=== modified file 'plugin/innobase/include/page0types.h'
--- plugin/innobase/include/page0types.h	2011-03-14 05:40:28 +0000
+++ plugin/innobase/include/page0types.h	2011-04-05 10:32:12 +0000
@@ -57,8 +57,9 @@
 
 /** Number of supported compressed page sizes */
 #define PAGE_ZIP_NUM_SSIZE (UNIV_PAGE_SIZE_SHIFT - PAGE_ZIP_MIN_SIZE_SHIFT + 2)
-#if PAGE_ZIP_NUM_SSIZE > (1 << PAGE_ZIP_SSIZE_BITS)
-# error "PAGE_ZIP_NUM_SSIZE > (1 << PAGE_ZIP_SSIZE_BITS)"
+#define PAGE_ZIP_NUM_SSIZE_MAX (UNIV_PAGE_SIZE_SHIFT_MAX - PAGE_ZIP_MIN_SIZE_SHIFT + 2)
+#if PAGE_ZIP_NUM_SSIZE_MAX > (1 << PAGE_ZIP_SSIZE_BITS)
+# error "PAGE_ZIP_NUM_SSIZE_MAX > (1 << PAGE_ZIP_SSIZE_BITS)"
 #endif
 
 /** Compressed page descriptor */
@@ -99,7 +100,7 @@
 typedef struct page_zip_stat_struct page_zip_stat_t;
 
 /** Statistics on compression, indexed by page_zip_des_struct::ssize - 1 */
-extern page_zip_stat_t page_zip_stat[PAGE_ZIP_NUM_SSIZE - 1];
+extern page_zip_stat_t page_zip_stat[PAGE_ZIP_NUM_SSIZE_MAX - 1];
 
 /**********************************************************************//**
 Write the "deleted" flag of a record on a compressed page.  The flag must

=== modified file 'plugin/innobase/include/page0zip.ic'
--- plugin/innobase/include/page0zip.ic	2010-12-18 04:43:40 +0000
+++ plugin/innobase/include/page0zip.ic	2011-04-05 10:32:12 +0000
@@ -174,12 +174,6 @@
 	ut_ad(ut_is_2pow(zip_size));
 	ut_ad(comp || !zip_size);
 
-#if UNIV_PAGE_SIZE > REC_MAX_DATA_SIZE
-	if (UNIV_UNLIKELY(rec_size >= REC_MAX_DATA_SIZE)) {
-		return(TRUE);
-	}
-#endif
-
 	if (UNIV_UNLIKELY(zip_size)) {
 		ut_ad(comp);
 		/* On a compressed page, there is a two-byte entry in

=== modified file 'plugin/innobase/include/rem0rec.ic'
--- plugin/innobase/include/rem0rec.ic	2010-12-18 04:43:40 +0000
+++ plugin/innobase/include/rem0rec.ic	2011-04-05 10:32:12 +0000
@@ -269,7 +269,7 @@
 	}
 
 	if (UNIV_EXPECT(comp, REC_OFFS_COMPACT)) {
-#if UNIV_PAGE_SIZE <= 32768
+
 		/* Note that for 64 KiB pages, field_value can 'wrap around'
 		and the debug assertion is not valid */
 
@@ -279,12 +279,13 @@
 		the expression could be written simpler as
 		(int16_t) field_value + ut_align_offset(...) < UNIV_PAGE_SIZE
 		*/
-		ut_ad((field_value >= 32768
-		       ? field_value - 65536
-		       : field_value)
-		      + ut_align_offset(rec, UNIV_PAGE_SIZE)
-		      < UNIV_PAGE_SIZE);
-#endif
+		ut_ad((UNIV_PAGE_SIZE > 32768)
+		      || ((field_value >= 32768
+			   ? field_value - 65536
+			   : field_value)
+			  + ut_align_offset(rec, UNIV_PAGE_SIZE)
+			  < UNIV_PAGE_SIZE));
+
 		/* There must be at least REC_N_NEW_EXTRA_BYTES + 1
 		between each record. */
 		ut_ad((field_value > REC_N_NEW_EXTRA_BYTES
@@ -337,7 +338,7 @@
 	field_value = mach_read_from_2(rec - REC_NEXT);
 
 	if (UNIV_EXPECT(comp, REC_OFFS_COMPACT)) {
-#if UNIV_PAGE_SIZE <= 32768
+
 		/* Note that for 64 KiB pages, field_value can 'wrap around'
 		and the debug assertion is not valid */
 
@@ -347,12 +348,13 @@
 		the expression could be written simpler as
 		(int16_t) field_value + ut_align_offset(...) < UNIV_PAGE_SIZE
 		*/
-		ut_ad((field_value >= 32768
-		       ? field_value - 65536
-		       : field_value)
+		ut_ad((UNIV_PAGE_SIZE > 32768)
+		      || (field_value >= 32768
+			  ? field_value - 65536
+			  : field_value)
 		      + ut_align_offset(rec, UNIV_PAGE_SIZE)
 		      < UNIV_PAGE_SIZE);
-#endif
+
 		if (UNIV_UNLIKELY(field_value == 0)) {
 
 			return(0);

=== modified file 'plugin/innobase/include/trx0sys.h'
--- plugin/innobase/include/trx0sys.h	2011-03-14 05:40:28 +0000
+++ plugin/innobase/include/trx0sys.h	2011-04-05 10:32:12 +0000
@@ -455,9 +455,6 @@
 /** Contents of TRX_SYS_MYSQL_LOG_MAGIC_N_FLD */
 #define TRX_SYS_MYSQL_LOG_MAGIC_N	873422344
 
-#if UNIV_PAGE_SIZE < 4096
-# error "UNIV_PAGE_SIZE < 4096"
-#endif
 /** The offset of the MySQL replication info in the trx system header;
 this contains the same fields as TRX_SYS_MYSQL_LOG_INFO below */
 #define TRX_SYS_MYSQL_MASTER_LOG_INFO	(UNIV_PAGE_SIZE - 2000)

=== modified file 'plugin/innobase/include/univ.i'
--- plugin/innobase/include/univ.i	2011-02-17 00:14:13 +0000
+++ plugin/innobase/include/univ.i	2011-04-05 10:32:12 +0000
@@ -315,10 +315,12 @@
 			========================
 */
 
-/* The 2-logarithm of UNIV_PAGE_SIZE: */
-#define UNIV_PAGE_SIZE_SHIFT	14
-/* The universal page size of the database */
-#define UNIV_PAGE_SIZE		(1 << UNIV_PAGE_SIZE_SHIFT)
+/* The 2-logarithm of UNIV_PAGE_SIZE_MAX: */
+#define UNIV_PAGE_SIZE_SHIFT_MAX	14
+#define UNIV_PAGE_SIZE_SHIFT    srv_page_size_shift
+/* The upper bound of universal page size of the database */
+#define UNIV_PAGE_SIZE_MAX      (1 << UNIV_PAGE_SIZE_SHIFT_MAX)
+#define UNIV_PAGE_SIZE		srv_page_size
 
 /* Maximum number of parallel threads in a parallelized operation */
 #define UNIV_MAX_PARALLELISM	32
@@ -434,7 +436,7 @@
 stored part of the field in the tablespace. The length field then
 contains the sum of the following flag and the locally stored len. */
 
-#define UNIV_EXTERN_STORAGE_FIELD (UNIV_SQL_NULL - UNIV_PAGE_SIZE)
+#define UNIV_EXTERN_STORAGE_FIELD (UNIV_SQL_NULL - UNIV_PAGE_SIZE_MAX)
 
 /* Some macros to improve branch prediction and reduce cache misses */
 #if defined(__GNUC__) && (__GNUC__ > 2) && ! defined(__INTEL_COMPILER)
@@ -537,4 +539,8 @@
 	UNIV_MEM_ALLOC(addr, size);			\
 } while (0)
 
+/* The 2-logarithm of srv_page_size */
+extern ulint    srv_page_size_shift;
+/* The universal page size of the database */
+extern ulint    srv_page_size;
 #endif

=== modified file 'plugin/innobase/include/ut0ut.h'
--- plugin/innobase/include/ut0ut.h	2011-03-14 05:40:28 +0000
+++ plugin/innobase/include/ut0ut.h	2011-07-30 12:19:38 +0000
@@ -172,7 +172,7 @@
 @param n	in: number to round down
 @param m	in: alignment, must be a power of two
 @return		n rounded down to the biggest possible integer multiple of m */
-#define ut_2pow_round(n, m) ((n) & ~((m) - 1))
+#define ut_2pow_round(n, m) ((n) & ~(typeof(n))((m) - 1))
 /** Align a number down to a multiple of a power of two.
 @param n	in: number to round down
 @param m	in: alignment, must be a power of two
@@ -184,7 +184,7 @@
 @param n	in: number to round up
 @param m	in: alignment, must be a power of two
 @return		n rounded up to the smallest possible integer multiple of m */
-#define ut_calc_align(n, m) (((n) + ((m) - 1)) & ~((m) - 1))
+#define ut_calc_align(n, m) (((n) + ((m) - 1)) & ~(typeof(n))((m) - 1))
 /*************************************************************//**
 Calculates fast the 2-logarithm of a number, rounded upward to an
 integer.

=== modified file 'plugin/innobase/log/log0log.cc'
--- plugin/innobase/log/log0log.cc	2011-04-07 05:47:57 +0000
+++ plugin/innobase/log/log0log.cc	2011-07-30 09:20:56 +0000
@@ -612,7 +612,8 @@
 
 	offset = (gr_lsn_size_offset + difference) % group_size;
 
-	ut_a(offset < (((ib_int64_t) 1) << 32)); /* offset must be < 4 GB */
+	/* Offset must be < 4 GB on 32 bit systems */
+	ut_a((sizeof(ulint) == 4) || (offset < (((ib_int64_t) 1) << 32)));
 
 	/* fprintf(stderr,
 	"Offset is %lu gr_lsn_offset is %lu difference is %lu\n",
@@ -1209,6 +1210,9 @@
 	/* Wipe over possible label of ibbackup --restore */
 	memcpy(buf + LOG_FILE_WAS_CREATED_BY_HOT_BACKUP, "    ", 4);
 
+	mach_write_to_4(buf + LOG_FILE_OS_FILE_LOG_BLOCK_SIZE,
+			srv_log_block_size);
+
 	dest_offset = nth_file * group->file_size;
 
 #ifdef UNIV_DEBUG
@@ -1802,9 +1806,7 @@
 	ulint		i;
 
 	ut_ad(mutex_own(&(log_sys->mutex)));
-#if LOG_CHECKPOINT_SIZE > OS_FILE_LOG_BLOCK_SIZE
-# error "LOG_CHECKPOINT_SIZE > OS_FILE_LOG_BLOCK_SIZE"
-#endif
+	ut_a(LOG_CHECKPOINT_SIZE <= OS_FILE_LOG_BLOCK_SIZE);
 
 	buf = group->checkpoint_buf;
 
@@ -1831,7 +1833,10 @@
 
 	mach_write_to_8(buf + LOG_CHECKPOINT_ARCHIVED_LSN, archived_lsn);
 #else /* UNIV_LOG_ARCHIVE */
-	mach_write_to_8(buf + LOG_CHECKPOINT_ARCHIVED_LSN, IB_ULONGLONG_MAX);
+	mach_write_to_8(buf + LOG_CHECKPOINT_ARCHIVED_LSN,
+			(ib_uint64_t)log_group_calc_lsn_offset(
+				log_sys->next_checkpoint_lsn, group));
+
 #endif /* UNIV_LOG_ARCHIVE */
 
 	for (i = 0; i < LOG_MAX_N_GROUPS; i++) {

=== modified file 'plugin/innobase/log/log0recv.cc'
--- plugin/innobase/log/log0recv.cc	2011-03-20 07:01:17 +0000
+++ plugin/innobase/log/log0recv.cc	2011-04-07 02:36:30 +0000
@@ -58,6 +58,8 @@
 UNIV_INTERN ibool	recv_replay_file_ops	= TRUE;
 #endif /* !UNIV_HOTBACKUP */
 
+#include <boost/scoped_array.hpp>
+
 #include <drizzled/errmsg_print.h>
 
 /** Log records are stored in the hash table in chunks at most of this size;
@@ -702,8 +704,22 @@
 
 			group->lsn = mach_read_from_8(
 				buf + LOG_CHECKPOINT_LSN);
-			group->lsn_offset = mach_read_from_4(
-				buf + LOG_CHECKPOINT_OFFSET);
+
+#ifdef UNIV_LOG_ARCHIVE
+#error "UNIV_LOG_ARCHIVE could not be enabled"
+#endif
+			{
+				ib_uint64_t tmp_lsn_offset = mach_read_from_8(
+					buf + LOG_CHECKPOINT_ARCHIVED_LSN);
+				if (sizeof(ulint) != 4
+				    && tmp_lsn_offset != IB_ULONGLONG_MAX) {
+					group->lsn_offset = (ulint) tmp_lsn_offset;
+			} else {
+					group->lsn_offset = mach_read_from_4(
+						buf + LOG_CHECKPOINT_OFFSET);
+				}
+			}
+
 			checkpoint_no = mach_read_from_8(
 				buf + LOG_CHECKPOINT_NO);
 
@@ -2885,6 +2901,7 @@
 	log_group_t*	max_cp_group;
 	log_group_t*	up_to_date_group;
 	ulint		max_cp_field;
+	ulint		log_hdr_log_block_size;
 	ib_uint64_t	checkpoint_lsn;
 	ib_uint64_t	checkpoint_no;
 	ib_uint64_t	old_scanned_lsn;
@@ -2894,7 +2911,10 @@
 	ib_uint64_t	archived_lsn;
 #endif /* UNIV_LOG_ARCHIVE */
 	byte*		buf;
-	byte		log_hdr_buf[LOG_FILE_HDR_SIZE];
+	boost::scoped_array<byte>	log_hdr_buf_base(
+		new byte[LOG_FILE_HDR_SIZE + OS_FILE_LOG_BLOCK_SIZE]);
+	byte*		log_hdr_buf
+		= (byte *)ut_align(log_hdr_buf_base.get(), OS_FILE_LOG_BLOCK_SIZE);
 	ulint		err;
 
 #ifdef UNIV_LOG_ARCHIVE
@@ -2919,7 +2939,7 @@
           drizzled::errmsg_printf(drizzled::error::INFO,
                                   "InnoDB: The user has set SRV_FORCE_NO_LOG_REDO on Skipping log redo.");
 
-		return(DB_SUCCESS);
+	  return(DB_SUCCESS);
 	}
 
 	recv_recovery_on = TRUE;
@@ -2978,6 +2998,21 @@
                  log_hdr_buf, max_cp_group);
 	}
 
+	log_hdr_log_block_size
+		= mach_read_from_4(log_hdr_buf + LOG_FILE_OS_FILE_LOG_BLOCK_SIZE);
+	if (log_hdr_log_block_size == 0) {
+		/* 0 means default value */
+		log_hdr_log_block_size = 512;
+	}
+	if (log_hdr_log_block_size != srv_log_block_size) {
+		drizzled::errmsg_printf(drizzled::error::ERROR,
+			   "InnoDB: Error: The block size of ib_logfile (%lu) "
+			   "is not equal to innodb_log_block_size.\n"
+			   "InnoDB: Error: Suggestion - Recreate log files.\n",
+			   log_hdr_log_block_size);
+		return(DB_ERROR);
+	}
+
 #ifdef UNIV_LOG_ARCHIVE
 	group = UT_LIST_GET_FIRST(log_sys->log_groups);
 

=== modified file 'plugin/innobase/mtr/mtr0log.cc'
--- plugin/innobase/mtr/mtr0log.cc	2010-12-24 00:04:05 +0000
+++ plugin/innobase/mtr/mtr0log.cc	2011-04-05 10:32:12 +0000
@@ -408,7 +408,7 @@
 	ptr += 2;
 
 	if (UNIV_UNLIKELY(offset >= UNIV_PAGE_SIZE)
-			|| UNIV_UNLIKELY(len + offset) > UNIV_PAGE_SIZE) {
+	    || UNIV_UNLIKELY(len + offset > UNIV_PAGE_SIZE)) {
 		recv_sys->found_corrupt_log = TRUE;
 
 		return(NULL);

=== modified file 'plugin/innobase/page/page0zip.cc'
--- plugin/innobase/page/page0zip.cc	2010-12-26 03:44:24 +0000
+++ plugin/innobase/page/page0zip.cc	2011-04-05 10:32:12 +0000
@@ -49,7 +49,7 @@
 
 #ifndef UNIV_HOTBACKUP
 /** Statistics on compression, indexed by page_zip_des_t::ssize - 1 */
-UNIV_INTERN page_zip_stat_t page_zip_stat[PAGE_ZIP_NUM_SSIZE - 1];
+UNIV_INTERN page_zip_stat_t page_zip_stat[PAGE_ZIP_NUM_SSIZE_MAX - 1];
 #endif /* !UNIV_HOTBACKUP */
 
 /* Please refer to ../include/page0zip.ic for a description of the
@@ -590,9 +590,6 @@
 #if PAGE_ZIP_DIR_SLOT_MASK & (PAGE_ZIP_DIR_SLOT_MASK + 1)
 # error "PAGE_ZIP_DIR_SLOT_MASK is not 1 less than a power of 2"
 #endif
-#if PAGE_ZIP_DIR_SLOT_MASK < UNIV_PAGE_SIZE - 1
-# error "PAGE_ZIP_DIR_SLOT_MASK < UNIV_PAGE_SIZE - 1"
-#endif
 		if (UNIV_UNLIKELY(rec_get_n_owned_new(rec))) {
 			offs |= PAGE_ZIP_DIR_SLOT_OWNED;
 		}

=== modified file 'plugin/innobase/row/row0merge.cc'
--- plugin/innobase/row/row0merge.cc	2010-12-26 23:42:28 +0000
+++ plugin/innobase/row/row0merge.cc	2011-04-05 10:32:12 +0000
@@ -98,7 +98,7 @@
 row_merge_block_t.  Thus, it must be able to hold one merge record,
 whose maximum size is the same as the minimum size of
 row_merge_block_t. */
-typedef byte	mrec_buf_t[UNIV_PAGE_SIZE];
+typedef byte	mrec_buf_t[UNIV_PAGE_SIZE_MAX];
 
 /** @brief Merge record in row_merge_block_t.
 

=== modified file 'plugin/innobase/srv/srv0srv.cc'
--- plugin/innobase/srv/srv0srv.cc	2011-04-11 21:43:24 +0000
+++ plugin/innobase/srv/srv0srv.cc	2011-07-30 09:20:56 +0000
@@ -266,6 +266,13 @@
 UNIV_INTERN ulint	srv_n_read_io_threads	= ULINT_MAX;
 UNIV_INTERN ulint	srv_n_write_io_threads	= ULINT_MAX;
 
+/* The universal page size of the database */
+UNIV_INTERN ulint	srv_page_size_shift	= 0;
+UNIV_INTERN ulint	srv_page_size		= 0;
+
+/* The log block size */
+UNIV_INTERN uint32_t	srv_log_block_size	= 0;
+
 /* User settable value of the number of pages that must be present
 in the buffer cache and accessed sequentially for InnoDB to trigger a
 readahead request. */

=== modified file 'plugin/innobase/srv/srv0start.cc'
--- plugin/innobase/srv/srv0start.cc	2011-04-11 21:43:24 +0000
+++ plugin/innobase/srv/srv0start.cc	2011-07-30 09:20:56 +0000
@@ -1332,9 +1332,11 @@
 	}
 #endif /* UNIV_LOG_ARCHIVE */
 
-	if (srv_n_log_files * srv_log_file_size >= 262144) {
-          drizzled::errmsg_printf(drizzled::error::ERROR,
-                                  "InnoDB: Error: combined size of log files must be < 4 GB");
+	if (sizeof(ulint) == 4
+	    && srv_n_log_files * srv_log_file_size
+	    >= (1UL << (32 - UNIV_PAGE_SIZE_SHIFT))) {
+		drizzled::errmsg_printf(drizzled::error::ERROR,
+				  "InnoDB: Error: combined size of log files must be < 4 GB on 32-bit systems\n");
 
 		return(DB_ERROR);
 	}
@@ -1343,7 +1345,7 @@
 
 	for (i = 0; i < srv_n_data_files; i++) {
 #ifndef __WIN__
-		if (sizeof(off_t) < 5 && srv_data_file_sizes[i] >= 262144) {
+		if (sizeof(off_t) < 5 && srv_data_file_sizes[i] >= (1UL << (32 - UNIV_PAGE_SIZE_SHIFT))) {
                   drizzled::errmsg_printf(drizzled::error::ERROR,
                                           "InnoDB: Error: file size must be < 4 GB with this MySQL binary and operating system combination,"
                                           " in some OS's < 2 GB\n");

=== modified file 'plugin/innobase/xtrabackup/xtrabackup.cc'
--- plugin/innobase/xtrabackup/xtrabackup.cc	2011-06-01 20:26:02 +0000
+++ plugin/innobase/xtrabackup/xtrabackup.cc	2011-08-10 21:46:38 +0000
@@ -38,6 +38,7 @@
 #include "ha_prototypes.h"
  //#define XTRABACKUP_TARGET_IS_PLUGIN
 #include <boost/program_options.hpp>
+#include <boost/scoped_array.hpp>
 #include <boost/scoped_ptr.hpp>
 
 typedef drizzled::charset_info_st CHARSET_INFO;
@@ -2786,7 +2787,8 @@
 	log_group_t*	max_cp_group;
 	ulint		max_cp_field;
 	byte*		buf;
-	byte		log_hdr_buf_[LOG_FILE_HDR_SIZE + OS_FILE_LOG_BLOCK_SIZE];
+	boost::scoped_array<byte>	log_hdr_buf_(
+		new byte[LOG_FILE_HDR_SIZE + OS_FILE_LOG_BLOCK_SIZE]);
 	byte*		log_hdr_buf;
 	ulint		err;
 
@@ -2796,7 +2798,8 @@
 	os_thread_id_t log_copying_thread_id;
 	datafiles_iter_t *it;
 
-	log_hdr_buf = (unsigned char*)ut_align(log_hdr_buf_, OS_FILE_LOG_BLOCK_SIZE);
+	log_hdr_buf = (byte*)ut_align(log_hdr_buf_.get(),
+				      OS_FILE_LOG_BLOCK_SIZE);
 
 	/* log space */
 	//space = UT_LIST_GET_NEXT(space_list, UT_LIST_GET_FIRST(f_system->space_list));
@@ -2882,8 +2885,9 @@
 				0, 0, LOG_FILE_HDR_SIZE);
 	} else {
 		/* Stream */
-		if (write(fileno(stdout), log_hdr_buf, LOG_FILE_HDR_SIZE)
-				== LOG_FILE_HDR_SIZE) {
+		if ((write(fileno(stdout), log_hdr_buf, LOG_FILE_HDR_SIZE)
+			- LOG_FILE_HDR_SIZE) == 0)
+		{
 			success = TRUE;
 		} else {
 			success = FALSE;

